@@ -7,8 +7,8 @@ from contextlib import closing
 from hashlib import md5
 from urlparse import urlparse
 from StringIO import StringIO
-import binascii, gzip, os, struct, weakref
-import ConfigParser 
+import binascii, gzip, os, shutil, struct, weakref
+import ConfigParser
 
 from downloader import Downloader, atomic_write
 
@@ -28,12 +28,6 @@ def set_spring_dir(path):
 	package_dir = os.path.join(spring_dir, 'packages')
 	content_dir = os.path.join(spring_dir, 'rapid')
 
-if os.name == 'posix':
-	set_spring_dir(os.path.expanduser('~/.spring'))
-#FIXME: elif os.name =='nt':
-else:
-	raise NotImplementedError('Unknown OS: %s' % os.name)
-
 ################################################################################
 
 def mkdir(path):
@@ -536,8 +530,10 @@ class Package(object):
 			if not self.can_be_installed:
 				raise DependencyException()
 			self.download_files(self.missing_files, progress)
-			#FIXME: Windows support
-			os.link(self.cache_file, self.installed_path)
+			if hasattr(os, 'link'):
+				os.link(self.cache_file, self.installed_path)
+			else:
+				shutil.copy(self.cache_file, self.installed_path)
 			if progress:
 				progress(progress.maximum())
 
@@ -583,10 +579,10 @@ class File(object):
 		# Get the physical path to the file in the pool.
 		hex = binascii.hexlify(md5)
 		pool_path = os.path.join(pool_dir, hex[:2], hex[2:]) + '.gz'
-	
+
 		# pool_path identifies the pool file, but name may differ per package.
 		key = (pool_path, name)
-	
+
 		# If we got it already, return flyweight File object.
 		if key in File.__files:
 			f = File.__files[key]
@@ -614,7 +610,7 @@ class File(object):
 
 ################################################################################
 
-import unittest, shutil
+import unittest
 from downloader import MockDownloader
 
 
